/*
  Implements a simple interface to the time and alarm function of the PCF8583 RTC chip

  Works around the device's limited year storage by keeping the year in the
  first two bytes of user accessible storage

  Copyright (c) 2013, Lumir Vanek
  
  Based on previous work of:
  -------------------------
  Erik DeBill https://github.com/edebill/PCF8583
  Xose Prez http://tinkerman.eldiariblau.net/

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


#include <Arduino.h>
#include <../Wire/Wire.h>
#include "PCF8583.h"



/*
 * Constructor
 *
 * Provide device address as a full 8 bit address (like the datasheet)
 */
PCF8583::PCF8583(int _deviceAddress) 
{
  deviceAddress = _deviceAddress >> 1;  // convert to 7 bit as Wire library requires
  Wire.begin();
}

/*
 * Get actual date-time from RTC to internal member variables
 */
void PCF8583::getTime()
{
	/*
	In original Erik DeBill's code, reading time starts with:
	Wire.send(0xC0);   // stop counting, don't mask
	
	... but datasheet says:

	When one of the counters is read (memory locations
	01 to 07), the contents of all counters are strobed into
	capture latches at the beginning of a read cycle. Therefore,
	faulty reading of the count during a carry condition is
	prevented.

	... so I removed stop counting. 
	*/

	Wire.beginTransmission(deviceAddress);
	Wire.write(0x02);
	Wire.endTransmission();
	
	Wire.requestFrom(deviceAddress, 5);
	second = bcdToByte(Wire.read());
	minute = bcdToByte(Wire.read());
	hour   = bcdToByte(Wire.read());

	byte incoming = Wire.read();				// year/date counter
	day    = bcdToByte(incoming & 0x3f);
	year   = (int) ((incoming >> 6) & 0x03);    // it will only hold 4 years...
	
	incoming  = Wire.read();
	month  = bcdToByte(incoming & 0x1f);		// 0 out the weekdays part
	weekday = incoming >> 5;

	//  but that's not all - we need to find out what the base year is
	//  so we can add the 2 bits we got above and find the real year
	Wire.beginTransmission(deviceAddress);
	Wire.write(LOCATION_BASE_YEAR);
	Wire.endTransmission();
	
	Wire.requestFrom(deviceAddress, 2);
	yearBase = 0;
	yearBase = Wire.read();
	yearBase = yearBase << 8;
	yearBase = yearBase | Wire.read();
	year = year + yearBase;
}

/*
   Set actual date-time to RTC from internal member variables
  
   Exapmle:
  
   rtc.year = 2013;
   rtc.month = 10;
   rtc.day = 1;
   rtc.hour = 16;
   rtc.minute = 40;
   rtc.second = 0;
   rtc.weekday = 2; // 0 = Sunday, 1 = Monday, etc. 
   rtc.setTime();

 */
void PCF8583::setTime()
{
	stop();
	
	Wire.beginTransmission(deviceAddress);
	Wire.write(0x02);
	Wire.write(intToBcd(second));
	Wire.write(intToBcd(minute));
	Wire.write(intToBcd(hour));
	Wire.write(((byte)(year % 4) << 6) | intToBcd(day));
	Wire.write(intToBcd(month) | (weekday << 5));
	Wire.endTransmission();

	Wire.beginTransmission(deviceAddress);
	Wire.write(LOCATION_BASE_YEAR);
	yearBase = year - year % 4;
	Wire.write(yearBase >> 8);
	Wire.write(yearBase & 0x00ff);
	Wire.endTransmission();

	start();
}

/* 
   Set a Clock alarm
  
   clockAlarmFunction: NO_CLOCK_ALARM, CLOCK_DAILY_ALARM, CLOCK_WEEKDAY_ALARM or CLOCK_DATED_ALARM
  
   Examples: 
  
   Fire interrupt daily at 17:30:00
   ================================
   rtc.secondAlarm = 0;
   rtc.minuteAlarm = 30;
   rtc.hourAlarm = 17;
   rtc.setClockAlarm(CLOCK_DAILY_ALARM);
  
   Fire interrupt Monday and Wednesday at 17:30:00
   ===============================================
   rtc.secondAlarm = 0;
   rtc.minuteAlarm = 30;
   rtc.hourAlarm = 17;
   rtc.weekdayAlarm = B000001010; // Set Bit1 for Monday and Bit3 for Wednesday
   rtc.setClockAlarm(CLOCK_WEEKDAY_ALARM); 
  
   Fire interrupt at specified date-time
   =====================================
   rtc.secondAlarm = 0;
   rtc.minuteAlarm = 30;
   rtc.hourAlarm = 17;
   rtc.dayAlarm = 20;
   rtc.monthAlarm = 1;
   rtc.setClockAlarm(CLOCK_DATED_ALARM); 
  
 */
void PCF8583::setClockAlarm(byte clockAlarmFunction)
{
	if(clockAlarmFunction == NO_CLOCK_ALARM)
	{
		byte control = read(LOCATION_ALARM_CONTROL);
		control &= 0x01001111;				// no interrupt, 'no clock alarm'
		write(LOCATION_ALARM_CONTROL, control);

		return;
	}
	
	byte control = B11000100;				// stop counting, don't mask, enable alarm control register, reset the alarm flag
	write(LOCATION_CONTROL, control);
	
	// Set Alarm time - Alarm time/date registers 0x09 - 0x0e
	Wire.beginTransmission(deviceAddress);
	Wire.write(0x09);						// set the register pointer to (0x09) - milliseconds
	Wire.write(0x00);						// set 00 at milliseconds
	Wire.write(intToBcd(secondAlarm));
	Wire.write(intToBcd(minuteAlarm));
	Wire.write(intToBcd(hourAlarm));
	
	// 0x0D - alarm date
	if(clockAlarmFunction == CLOCK_DATED_ALARM)	
		Wire.write(intToBcd(dayAlarm));
	else
		Wire.write(0x00);					// for 'daily alarm' or 'weekday alarm' Set 00 at date
	
	// 0x0E - alarm month
	if(clockAlarmFunction == CLOCK_WEEKDAY_ALARM)	
		Wire.write(weekdayAlarm);
	else if(clockAlarmFunction == CLOCK_DATED_ALARM)
		Wire.write(intToBcd(monthAlarm));

	Wire.endTransmission();

	byte controlAlarm =  B10000000;			// enable alarm interrupt
	controlAlarm |= ((clockAlarmFunction & B00110000));
    write(LOCATION_ALARM_CONTROL, controlAlarm);

	control &= B00111111;					// start counting
    write(LOCATION_CONTROL, control);
}

/* 
   Set a Timer alarm
   
   timer: timer value in range 0 - 99
   timerFunction: NO_TIMER, TIMER_HUNDRS_OF_A_SEC, TIMER_SECONDS, TIMER_MINUTES, TIMER_HOURS or TIMER_DAYS
  
   Example: 
  
   Fire interrupt every 8 seconds
   ================================
   rtc.setTimerAlarm(8, TIMER_SECONDS);
  
 */
void PCF8583::setTimerAlarm(byte timer, byte timerFunction)
{
	if(timerFunction == NO_TIMER)
	{
		byte control = read(LOCATION_ALARM_CONTROL);
		control &= 0x01111000;				// no interrupt, 'no timer'
		write(LOCATION_ALARM_CONTROL, control);

		return;
	}

	byte control = B11000100;				// stop counting, don't mask, enable alarm control register, reset the timer flag
	write(LOCATION_CONTROL, control);
	
	write(LOCATION_ALARM_TIMER, intToBcd(timer));
	write(0x07, 0x00);						// reset timer counter

	byte controlAlarm = B11001000;			// set 'timer alarm' on, timer interrupt on
	controlAlarm |= (timerFunction & B00000111);
	write(LOCATION_ALARM_CONTROL, controlAlarm); 
	
	control &= B00111111;					// start counting
    write(LOCATION_CONTROL, control);
}

/*
 * Restart alarm, must be called to re-enable alarm after clock alarm interrupt is handled
 */
void PCF8583::restartClockAlarm()
{
	byte control = B00000100;				// enable alarm control register, reset the alarm and timer flags
	write(LOCATION_CONTROL, control);
}

/*
 * Restart alarm and reset timer counter, must be called to re-enable alarm after timer alarm interrupt is handled
 */
void PCF8583::restartTimerAlarm()
{
	restartClockAlarm();
	write(0x07, 0x00);		// reset timer counter
}

/*
 * Write given byte to NVRAM in PCF8583
 *
 * Note that address range 0x00 - 0x11 is reserved for RTC functionality !
 */
void PCF8583::write(byte nvramAddress, byte value) 
{
	Wire.beginTransmission(deviceAddress);
	Wire.write(nvramAddress);
	Wire.write(value);
	Wire.endTransmission();
}

/*
 * Read byte from NVRAM in PCF8583
 */
byte PCF8583::read(byte nvramAddress) 
{
	Wire.beginTransmission(deviceAddress);
	Wire.write(nvramAddress);
	Wire.endTransmission();
	
	Wire.requestFrom(deviceAddress, 1);
	return Wire.read();
}

// Private methods

void PCF8583::stop() 
{
	byte control = read(LOCATION_CONTROL);
	control |= 0x80;
	write(LOCATION_CONTROL, control);
}

void PCF8583::start() 
{
	byte control = read(LOCATION_CONTROL);
	control &= 0x7F;
	write(LOCATION_CONTROL, control);
}

int PCF8583::bcdToByte(byte bcd)
{
	return ((bcd >> 4) * 10) + (bcd & 0x0f);
}

byte PCF8583::intToBcd(int in)
{
	return ((in / 10) << 4) + (in % 10);
}
