/*
  Implements a simple interface to the time and alarm function of the PCF8583 RTC chip

  Works around the device's limited year storage by keeping the year in the
  first two bytes of user accessible storage

  Copyright (c) 2013, Lumir Vanek
  
  Based on previous work of:
  -------------------------
  Erik DeBill https://github.com/edebill/PCF8583
  Xose Prez http://tinkerman.eldiariblau.net/

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*

   Usage:
      PCF8583 rtc(0xA0); // or 0xA2 if A0 pin connected to Vdd
      rtc.getTime();

      Serial.print("year: ");
      Serial.println(rtc.year);


      rtc.hour = 14;
      rtc.minute = 30
      rtc.second = 0
      rtc.year = 2009
      rtc.month = 9
      rtc.day = 12
      rtc.setTime();

*/

#ifndef PCF8583_H
#define PCF8583_H

#include <Arduino.h>
#include <../Wire/Wire.h>


#define LOCATION_CONTROL		0x00
#define LOCATION_ALARM_CONTROL	0x08
#define LOCATION_ALARM_TIMER	0x0F
#define LOCATION_BASE_YEAR		0x10	// Because PCF8583 only hold 4 years - we uses this address (two bytes) in NVRAM to store base year 

// Clock alarm modes - Bits 4-5 in RTC Alarm control register
#define NO_CLOCK_ALARM			0x00
#define CLOCK_DAILY_ALARM		0x10
#define CLOCK_WEEKDAY_ALARM		0x20
#define CLOCK_DATED_ALARM		0x30

// Timer alarm modes - Bits 0-3 in RTC Alarm control register
#define NO_TIMER				0x00
#define TIMER_HUNDRS_OF_A_SEC	0x01
#define TIMER_SECONDS			0x02
#define TIMER_MINUTES			0x03
#define TIMER_HOURS				0x04
#define TIMER_DAYS				0x05


class PCF8583 
{
public:
	// Clock
    int second;
    int minute;
    int hour;

	// Calendar
    int day;
    int month;
    int year;
	int weekday;      // 0 = Sunday, 1 = Monday, etc. 
    
	// Alarm
	int secondAlarm;
    int minuteAlarm;
    int hourAlarm;
    int dayAlarm;
	int monthAlarm;
	int weekdayAlarm;  // RTC memory location 0Eh (alarm_weekday/month): Bit0=Sunday, ... Bit6=Saturday, Bit7=Not used

	// Constructor
    PCF8583(int _deviceAddress);

	
	int getAddress() { return deviceAddress; }

	void getTime();
    void setTime();

	void setClockAlarm(byte clockAlarmFunction);
	void setTimerAlarm(byte timer, byte timerFunction);
	
	void restartClockAlarm();
	void restartTimerAlarm();

	void write(byte nvramAddress, byte value);
	byte read(byte nvramAddress);

private:
	int deviceAddress; // Converted to 7 bit as Wire library requires
	int yearBase;

	void stop();
	void start();

	int bcdToByte(byte bcd);
	byte intToBcd(int in);
};


#endif  //PCF8583_H
